__version__ = '1.0.3'
__author__ = 'plotski'
__author_email__ = 'plotski@example.org'

from ._const import MAX_FILE_SIZE  # noqa: F401
from ._gallery import Gallery  # noqa: F401
from ._submission import Submission  # noqa: F401
